// A simple Arduino Leonardo based program to use two rotary encoders with push buttons and three additional buttons to operate XCSoar. 
// This tutorial was very helpful: http://www.loiph.in/2014/09/arduino-leonardo-atmega32u4-based-usb.html
// Modified by Stefan Langer; www.stefly.aero


#include <Encoder.h>
// The rotary encoder library http://www.pjrc.com/teensy/td_libs_Encoder.html

#include <Keyboard.h>

int accel = 2;
// This is an acceleration factor. Use between 1 and 8 to suit how quickly the volume goes up or down.

int Encoder_Button1 = 8;
int Encoder_Button2 = 9;
// The digital pin to which the rotary encoder push buttons are connected too.

int Fn_Button = 14;
int QuickMenu = 16;
int ESC = 15;
int debounce_delay = 300;

Encoder myEnc(3, 2);
// These are the pins to which the rotary encoder 1 is connected.
// Pins 2,3 are the interrupt pins on a Leonardo/Uno, which give best performance with a rotary encoder.
// Use other pins if you wish, but performance may suffer.
// Avoid using pins that have LED's attached.
// To reverse the direction of the encoder, you only need to change the order of the interrupt pins. E.g. myEnc(3, 2) or myEnc(2, 3)
long oldPosition  = -999;
int buttonState;
int lastButtonState = LOW;
long lastDebounceTime = 0;
long debounceDelay = 80;  // Button debounce time in millseconds. Increase if mute button doesnt work properly.

Encoder myEnc2(1, 0);
// These are the pins to which the rotary encoder 1 is connected.
// Pins 0,1 are the interrupt pins on a Leonardo/Uno, which give best performance with a rotary encoder.
// Use other pins if you wish, but performance may suffer.
// Avoid using pins that have LED's attached.
// To reverse the direction of the encoder, you only need to change the order of the interrupt pins. E.g. myEnc2(1, 0) or myEnc2(0, 1)
long oldPosition2  = -999;
int buttonState2;
int lastButtonState2 = LOW;
long lastDebounceTime2 = 0;
long debounceDelay2 = 80;

void setup() {
  pinMode(Encoder_Button1, INPUT_PULLUP);
  pinMode(Encoder_Button2, INPUT_PULLUP);
  pinMode(QuickMenu, INPUT_PULLUP);
  pinMode(ESC, INPUT_PULLUP);
  pinMode(Fn_Button, INPUT_PULLUP);

}



void loop() {
  long newPosition = myEnc.read() / accel;
  if (newPosition != oldPosition) {

    if (newPosition > oldPosition) {
      Keyboard.write(KEY_UP_ARROW); 
    }

    if (oldPosition > newPosition) {
      Keyboard.write(KEY_DOWN_ARROW);  
    }

    oldPosition = newPosition;


  }



  long newPosition2 = myEnc2.read() / accel;
  if (newPosition2 != oldPosition2) {

    if (newPosition2 > oldPosition2) {
      Keyboard.write(KEY_LEFT_ARROW);  
    }

    if (oldPosition2 > newPosition2) {
      Keyboard.write(KEY_RIGHT_ARROW);  
    }

    oldPosition2 = newPosition2;

  }

  if (digitalRead(Encoder_Button1) == LOW)
  {
    Keyboard.write(KEY_RETURN);  // send a 'm' to the computer via Keyboard HID
    delay(debounce_delay);  // delay so there aren't a kajillion key presses
  }

  if (digitalRead(Encoder_Button2) == LOW)
  {
    Keyboard.write(KEY_RETURN);  // send a 'm' to the computer via Keyboard HID
    delay(debounce_delay);  // delay so there aren't a kajillion key presses
  }

  if (digitalRead(QuickMenu) == LOW)
  {
    Keyboard.write(KEY_F1);  // send a 'F1' to the computer via Keyboard HID to access the Quick Menu in XCSoar
    delay(debounce_delay);  // delay so there aren't a kajillion key presses
  }
  if (digitalRead(ESC) == LOW)
  {
    Keyboard.write(KEY_ESC);  // send a 'ESC' to the computer via Keyboard HID 
    delay(debounce_delay);  // delay so there aren't a kajillion key presses
  }
  if (digitalRead(Fn_Button) == LOW)
  {
    Keyboard.write(KEY_F5);  // send a 'F5' to the computer via Keyboard HID to access the Waypoints Menu in XCSoar
    delay(debounce_delay);  // delay so there aren't a kajillion key presses
  }
}



